package hdm.mi.growbros.config;

import com.fasterxml.jackson.core.JacksonException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.repositories.PlantRepository;
import hdm.mi.growbros.util.FileReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;

import java.util.List;

@Configuration
public class Seeding {
    private static final Logger log = LoggerFactory.getLogger(Seeding.class);

    @Bean
    @Profile("!test")
    public CommandLineRunner plants(PlantRepository plantRepository, FileReader fileReader) {
        return (args) -> {
            if (plantRepository.count() > 0) {
                log.info("Plants table is not empty, seeding will be skipped.");
                log.info("If you wish to seed the database, manually clear the plants table and re-run the application.");
                return;
            }

            String fileName = "plants/default-plants.json";
            log.info("Trying to seed database with file: '{}'", fileName);
            String fileContent = fileReader.getResourceFileAsString(fileName);

            try {
                final ObjectMapper objectMapper = new ObjectMapper();
                List<Plant> plants = objectMapper.readValue(fileContent, new TypeReference<>() {
                });
                log.info("Read {} plants", plants.size());
                plantRepository.saveAll(plants);
                log.info("Successfully seeded database");
            } catch (JacksonException e) {
                log.error(e.getMessage());
            }
        };
    }
}
