package hdm.mi.growbros.service;

import hdm.mi.growbros.models.dto.CustomPageDto;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.search.PlantSpecification;
import hdm.mi.growbros.repositories.PlantRepository;
import hdm.mi.growbros.models.search.SearchRequest;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

@Service
@RequiredArgsConstructor
public class PlantsService {
    private final static Logger log = LoggerFactory.getLogger(PlantsService.class);
    private final static int MAX_PAGE_SIZE = 100;

    private final PlantRepository plantRepository;
    private final PlantSpecification plantSpecification;

    public CustomPageDto<Plant> getPlants(int pageNumber, int pageSize) {
        pageNumber = Math.max(0, pageNumber);
        pageSize = Math.min(MAX_PAGE_SIZE, pageSize);

        final Pageable page = PageRequest.of(pageNumber, pageSize);
        List<Plant> pageContent = plantRepository.findAll(page).getContent();
        return new CustomPageDto<>(pageNumber, pageContent.size(), pageContent);
    }

    public Collection<Plant> getRandomPlants(int numberOfPlants) {
        List<Plant> allPlants = plantRepository.findAll();

        if (allPlants.size() <= numberOfPlants) {
            return allPlants;
        } else {
            List<Plant> randomPlants = new ArrayList<>();
            Random random = new Random();

            while (randomPlants.size() < numberOfPlants) {
                Plant randomPlant = allPlants.get(random.nextInt(allPlants.size()));

                if (!randomPlants.contains(randomPlant)) {
                    randomPlants.add(randomPlant);
                }
            }
            return randomPlants;
        }
    }

    public Collection<Plant> getPlantsSearchedFor(SearchRequest searchRequest) {
        Specification<Plant> spec = plantSpecification.withSearchCriteria(searchRequest);
        return plantRepository.findAll(spec);
    }

    public void createPlant(Plant plant) {
        log.info("Saving plant '{}' to repository.", plant.getName());
        plantRepository.save(plant);
    }
}
